#include <msp/gl/extensions/arb_enhanced_layouts.h>
#include <msp/gl/extensions/arb_explicit_attrib_location.h>
#include <msp/gl/extensions/arb_explicit_uniform_location.h>
#include <msp/gl/extensions/arb_gpu_shader5.h>
#include <msp/gl/extensions/arb_separate_shader_objects.h>
#include <msp/gl/extensions/arb_uniform_buffer_object.h>
#include <msp/gl/extensions/arb_vertex_shader.h>
#include <msp/gl/extensions/ext_framebuffer_multisample.h>
#include <msp/gl/extensions/ext_framebuffer_object.h>
#include <msp/gl/extensions/ext_gpu_shader4.h>
#include <msp/gl/extensions/ext_texture_array.h>
#include <msp/gl/extensions/ext_texture_filter_anisotropic.h>
#include <msp/gl/extensions/msp_clipping.h>
#include <msp/gl/extensions/nv_fbo_color_attachments.h>
#include "device.h"
#include "device_backend.h"
#include "gl.h"

using namespace std;

namespace Msp {
namespace GL {

OpenGLDevice::OpenGLDevice(Graphics::Window &wnd, const Graphics::GLOptions &opts):
	context(wnd, opts)
{ }

Graphics::GLOptions OpenGLDevice::create_default_options()
{
	Graphics::GLOptions opts;
	opts.gl_version_major = Graphics::GLOptions::LATEST_VERSION;
	opts.core_profile = true;
	return opts;
}

void OpenGLDevice::fill_info()
{
	DeviceInfo &info = static_cast<Device *>(this)->info;

	if(const char *gl_ver_ptr = reinterpret_cast<const char *>(glGetString(GL_VERSION)))
	{
		string gl_ver = gl_ver_ptr;
		if(!gl_ver.compare(0, 10, "OpenGL ES "))
			gl_ver.erase(0, 10);

		Version ver(gl_ver.substr(0, gl_ver.find(' ')));

		if(const char *force_ver_ptr = getenv("MSPGL_FORCE_VERSION"))
		{
			Version force_ver(force_ver_ptr);
			if(force_ver<ver)
				ver = force_ver;
		}

		info.api_version = ver;
	}

	DeviceLimits &lim = info.limits;
	glGetIntegerv(GL_MAX_CLIP_PLANES, reinterpret_cast<int *>(&lim.max_clip_planes));
	if(ARB_vertex_shader)
	{
		glGetIntegerv(GL_MAX_VERTEX_ATTRIBS, reinterpret_cast<int *>(&lim.max_vertex_attributes));
		glGetIntegerv(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, reinterpret_cast<int *>(&lim.max_texture_bindings));
	}
	if(EXT_framebuffer_object)
		glGetIntegerv(GL_MAX_COLOR_ATTACHMENTS, reinterpret_cast<int *>(&lim.max_color_attachments));
	if(EXT_framebuffer_multisample)
		glGetIntegerv(GL_MAX_SAMPLES, reinterpret_cast<int *>(&lim.max_samples));
	if(ARB_uniform_buffer_object)
	{
		glGetIntegerv(GL_MAX_UNIFORM_BUFFER_BINDINGS, reinterpret_cast<int *>(&lim.max_uniform_bindings));
		glGetIntegerv(GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT, reinterpret_cast<int *>(&lim.uniform_buffer_alignment));
	}
	if(EXT_texture_filter_anisotropic)
		glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY, &lim.max_anisotropy);

	SL::Features &feat = info.glsl_features;
	feat.target_api = get_backend_api();
	feat.glsl_version = get_glsl_version();
	feat.arb_enhanced_layouts = ARB_enhanced_layouts;
	feat.arb_explicit_attrib_location = ARB_explicit_attrib_location;
	feat.arb_explicit_uniform_location = ARB_explicit_uniform_location;
	feat.arb_gpu_shader5 = ARB_gpu_shader5;
	feat.arb_separate_shader_objects = ARB_separate_shader_objects;
	feat.arb_uniform_buffer_object = ARB_uniform_buffer_object;
	feat.ext_gpu_shader4 = EXT_gpu_shader4;
	feat.ext_texture_array = EXT_texture_array;
	feat.uniform_binding_range = lim.max_uniform_bindings;
	feat.texture_binding_range = lim.max_texture_bindings;

	state.bound_tex_targets.resize(lim.max_texture_bindings);
	state.bound_uniform_blocks.resize(lim.max_uniform_bindings);
}

} // namespace GL
} // namespace Msp
