#ifndef MSP_GL_EXTENSION_H_
#define MSP_GL_EXTENSION_H_

#include <string>
#include "backend.h"

namespace Msp {
namespace GL {

enum GLProfile
{
	CORE_PROFILE,
	COMPATIBILITY_PROFILE
};


/**
Holds metadata about an extension.  Evaluates to true if the extension is
supported.
*/
class Extension
{
public:
	enum SupportLevel
	{
		UNSUPPORTED,
		EXTENSION,
		CORE
	};

	typedef SupportLevel (*InitFunc)();

private:
	const char *name;
	InitFunc init_func;
	mutable bool init_done;
	mutable SupportLevel support;

public:
	Extension(const char *, InitFunc);

	const char *get_name() const { return name; }
	operator bool() const;
};


struct Require
{
	Require(const Extension &);
};


typedef void ExtFunc();

/** Checks for extension support.  Only intended for internal use. */
bool is_supported(const std::string &);

/** Checks for OpenGL version support.  Only intended for internal use. */
bool is_supported(const Version &, const Version & = Version());

/** Indicates whether an extension has been disabled, either explicitly through
the MSPGL_DISABLE_EXTENSIONS environment variable or implicitly as a workaround
for a driver bug.  Only intended for internal use. */
bool is_disabled(const std::string &);

/** Returns the OpenGL profile for the active context. */
GLProfile get_gl_profile();

/** Returns the GLSL version number, as reported by the implementation. */
const Version &get_glsl_version();

/** Returns the address of an extension function.  Only indended for internal
use. */
ExtFunc *get_proc_address(const std::string &);

} // namespace GL
} // namespace Msp

#endif
