#include <msp/gl/extensions/arb_depth_buffer_float.h>
#include <msp/gl/extensions/arb_depth_texture.h>
#include <msp/gl/extensions/arb_texture_float.h>
#include <msp/gl/extensions/arb_texture_rg.h>
#include <msp/gl/extensions/ext_texture_srgb.h>
#include <msp/gl/extensions/oes_required_internalformat.h>
#include <msp/gl/extensions/oes_texture_stencil8.h>
#include "gl.h"
#include "pixelformat.h"
#include "pixelformat_backend.h"

using namespace std;

namespace {

int swizzle_orders[] =
{
	GL_RED, GL_GREEN, GL_BLUE, GL_ALPHA,
	GL_RED, GL_RED, GL_RED, GL_ONE,
	GL_RED, GL_RED, GL_RED, GL_GREEN,
	GL_BLUE, GL_GREEN, GL_RED, GL_ALPHA,
	GL_RED, GL_GREEN, GL_BLUE, GL_ONE
};

}

namespace Msp {
namespace GL {

ComponentSwizzle get_required_swizzle(PixelComponents comp)
{
	switch(comp)
	{
	case BGR: return RGB_TO_BGR;
	case BGRA: return RGB_TO_BGR;
	case LUMINANCE: return R_TO_LUMINANCE;
	case LUMINANCE_ALPHA: return RG_TO_LUMINANCE_ALPHA;
	default: return NO_SWIZZLE;
	}
}

void require_pixelformat(PixelFormat pf)
{
	/* TODO These checks are only accurate for textures.  On OpenGL ES some
	formats are allowed for render buffers earlier than textures.  In particular
	it's possible to create a 16-bit depth renderbuffer on OpenGL ES 2.0 but
	depth textures are only available with 3.0 or the OES_depth_texture
	extension.*/
	switch(pf)
	{
	case RGB8:
	case RGBA8:
		{ static Require _req(OES_required_internalformat); }
		break;
	case R8:
	case RG8:
		{ static Require _req(ARB_texture_rg); }
		break;
	case R16F:
	case R32F:
	case RG16F:
	case RG32F:
		{ static Require _req(ARB_texture_rg); }
		{ static Require _req(ARB_texture_float); }
		break;
	case RGB16F:
	case RGB32F:
	case RGBA16F:
	case RGBA32F:
		{ static Require _req(ARB_texture_float); }
		break;
	case SRGB8:
	case SRGB8_ALPHA8:
		{ static Require _req(EXT_texture_sRGB); }
		break;
	case DEPTH_COMPONENT16:
	case DEPTH_COMPONENT24:
	case DEPTH_COMPONENT32:
		{ static Require _req(ARB_depth_texture); }
		{ static Require _req(OES_required_internalformat); }
		break;
	case DEPTH_COMPONENT32F:
		{ static Require _req(ARB_depth_buffer_float); }
		break;
	case STENCIL_INDEX8:
		{ static Require _req(OES_texture_stencil8); }
		break;
	default:
		throw invalid_argument("require_pixelformat");
	}
}

unsigned get_gl_components(PixelComponents comp)
{
	switch(comp)
	{
	case RED: return GL_RED;
	case RG: return GL_RG;
	case RGB: return GL_RGB;
	case RGBA: return GL_RGBA;
	case DEPTH_COMPONENT: return GL_DEPTH_COMPONENT;
	case STENCIL_INDEX: return GL_STENCIL_INDEX;
	default: throw invalid_argument("get_gl_components");
	}
}

unsigned get_gl_pixelformat(PixelFormat pf)
{
	switch(pf)
	{
	case R8: return GL_R8;
	case R16F: return GL_R16F;
	case R32F: return GL_R32F;
	case RG8: return GL_RG8;
	case RG16F: return GL_RG16F;
	case RG32F: return GL_RG32F;
	case RGB8: return GL_RGB8;
	case RGB16F: return GL_RGB16F;
	case RGB32F: return GL_RGB32F;
	case RGBA8: return GL_RGBA8;
	case RGBA16F: return GL_RGBA16F;
	case RGBA32F: return GL_RGBA32F;
	case SRGB8: return GL_SRGB8;
	case SRGB8_ALPHA8: return GL_SRGB8_ALPHA8;
	case DEPTH_COMPONENT16: return GL_DEPTH_COMPONENT16;
	case DEPTH_COMPONENT24: return GL_DEPTH_COMPONENT24;
	case DEPTH_COMPONENT32: return GL_DEPTH_COMPONENT32;
	case DEPTH_COMPONENT32F: return GL_DEPTH_COMPONENT32F;
	case STENCIL_INDEX8: return GL_STENCIL_INDEX8;
	default: throw invalid_argument("get_gl_pixelformat");
	}
}

const int *get_gl_swizzle(ComponentSwizzle swiz)
{
	return swizzle_orders+4*swiz;
}

} // namespace GL
} // namespace Msp
