#include <msp/gl/extensions/arb_direct_state_access.h>
#include <msp/gl/extensions/arb_texture_storage.h>
#include <msp/gl/extensions/msp_texture1d.h>
#include "gl.h"
#include "texture1d.h"
#include "texture1d_backend.h"

namespace Msp {
namespace GL {

OpenGLTexture1D::OpenGLTexture1D():
	Texture(GL_TEXTURE_1D)
{
	static Require _req(MSP_texture1D);
}

void OpenGLTexture1D::allocate()
{
	const Texture1D &self = *static_cast<const Texture1D *>(this);

	if(!id)
		create();

	GLenum gl_fmt = get_gl_pixelformat(storage_fmt);
	if(ARB_texture_storage)
	{
		if(ARB_direct_state_access)
			glTextureStorage1D(id, self.levels, gl_fmt, self.width);
		else
		{
			bind_scratch();
			glTexStorage1D(target, self.levels, gl_fmt, self.width);
		}
	}
	else
	{
		bind_scratch();
		glTexParameteri(target, GL_TEXTURE_MAX_LEVEL, self.levels-1);
		GLenum comp = get_gl_components(get_components(storage_fmt));
		GLenum type = get_gl_type(get_component_type(storage_fmt));
		for(unsigned i=0; i<self.levels; ++i)
		{
			unsigned lv_size = self.get_level_size(i);
			glTexImage1D(target, i, gl_fmt, lv_size, 0, comp, type, 0);
		}
	}

	apply_swizzle();
}

void OpenGLTexture1D::sub_image(unsigned level, int x, unsigned wd, const void *data)
{
	GLenum comp = get_gl_components(get_components(storage_fmt));
	GLenum type = get_gl_type(get_component_type(storage_fmt));
	if(ARB_direct_state_access)
		glTextureSubImage1D(id, level, x, wd, comp, type, data);
	else
	{
		bind_scratch();
		glTexSubImage1D(target, level, x, wd, comp, type, data);
	}
}

size_t OpenGLTexture1D::get_data_size() const
{
	if(!id)
		return 0;

	const Texture1D &self = *static_cast<const Texture1D *>(this);

	size_t level_size = self.width*get_pixel_size(storage_fmt);
	size_t total_size = level_size;
	for(unsigned i=0; i<self.levels; ++i, level_size>>=2)
		total_size += level_size;
	return total_size;
}

} // namespace GL
} // namespace Msp
