#include <msp/gl/extensions/arb_direct_state_access.h>
#include <msp/gl/extensions/arb_texture_multisample.h>
#include <msp/gl/extensions/arb_texture_storage_multisample.h>
#include "texture2dmultisample.h"
#include "texture2dmultisample_backend.h"

namespace Msp {
namespace GL {

OpenGLTexture2DMultisample::OpenGLTexture2DMultisample():
	Texture(GL_TEXTURE_2D_MULTISAMPLE)
{
	static Require _req(ARB_texture_multisample);
}

void OpenGLTexture2DMultisample::allocate()
{
	const Texture2DMultisample &self = *static_cast<const Texture2DMultisample *>(this);

	if(!id)
		create();

	GLenum gl_fmt = get_gl_pixelformat(storage_fmt);
	if(ARB_texture_storage_multisample)
	{
		if(ARB_direct_state_access)
			glTextureStorage2DMultisample(id, self.samples, gl_fmt, self.width, self.height, false);
		else
		{
			bind_scratch();
			glTexStorage2DMultisample(target, self.samples, gl_fmt, self.width, self.height, false);
		}
	}
	else
	{
		bind_scratch();
		glTexImage2DMultisample(target, self.samples, gl_fmt, self.width, self.height, false);
	}
	apply_swizzle();
}

size_t OpenGLTexture2DMultisample::get_data_size() const
{
	const Texture2DMultisample &self = *static_cast<const Texture2DMultisample *>(this);
	return self.width*self.height*get_pixel_size(format)*self.samples;
}

} // namespace GL
} // namespace Msp
