#ifndef MSP_GL_VULKAN_SYNCHRONIZER_H_
#define MSP_GL_VULKAN_SYNCHRONIZER_H_

#include <cstdint>
#include <vector>
#include "handles.h"

namespace Msp {
namespace GL {

class Synchronizer
{
private:
	struct ImageAccess
	{
		VkImage image = 0;
		unsigned aspect;
		int layer = -1;
		unsigned current_layout;
		unsigned pending_layout;
	};

	struct BufferAccess
	{
		VkBuffer buffer = 0;
		std::size_t offset = 0;
		std::size_t size = 0;
		bool was_written = false;
		bool pending_write = false;
	};

	Device &device;
	std::vector<BufferAccess> buffer_accesses;
	std::vector<ImageAccess> image_accesses;

public:
	Synchronizer(Device &);

	void access(VkBuffer, std::size_t, std::size_t);
	void access(VkImage, unsigned, int, unsigned, bool);
	void reset();
	void barrier(VkCommandBuffer);

private:
	bool is_write_layout(unsigned);
};

} // namespace GL
} // namespace Msp

#endif
