#include "device.h"
#include "texture1d.h"
#include "texture1d_backend.h"
#include "vulkan.h"

using namespace std;

namespace Msp {
namespace GL {

VulkanTexture1D::VulkanTexture1D():
	Texture(VK_IMAGE_VIEW_TYPE_1D)
{ }

void VulkanTexture1D::fill_image_info(void *ii) const
{
	const Texture1D &self = *static_cast<const Texture1D *>(this);

	VkImageCreateInfo *image_info = static_cast<VkImageCreateInfo *>(ii);
	image_info->imageType = VK_IMAGE_TYPE_1D;
	image_info->extent.width = self.width;
	image_info->mipLevels = self.levels;
}

void VulkanTexture1D::sub_image(unsigned level, int x, unsigned wd, const void *data)
{
	const Texture1D &self = *static_cast<const Texture1D *>(this);

	unsigned level_size = self.get_level_size(level);
	synchronize(-1, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, (x==0 && wd==level_size));

	size_t data_size = wd*get_pixel_size(storage_fmt);
	void *staging = device.get_transfer_queue().prepare_transfer(data_size,
		[this, level, x, wd](VkCommandBuffer cmd_buf, VkBuffer staging_buf, size_t src_off){
			const VulkanFunctions &vk = device.get_functions();

			VkBufferImageCopy region = { };
			region.bufferOffset = src_off;
			region.imageSubresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
			region.imageSubresource.mipLevel = level;
			region.imageSubresource.baseArrayLayer = 0;
			region.imageSubresource.layerCount = 1;
			region.imageOffset = { x, 0, 0 };
			region.imageExtent = { wd, 1, 1 };
			vk.CmdCopyBufferToImage(cmd_buf, staging_buf, handle, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 1, &region);
		});

	const char *src = static_cast<const char *>(data);
	copy(src, src+data_size, static_cast<char *>(staging));
}

size_t VulkanTexture1D::get_data_size() const
{
	return 0;
}

} // namespace GL
} // namespace Msp
