#include "device.h"
#include "texture2d.h"
#include "texture2d_backend.h"
#include "vulkan.h"

using namespace std;

namespace Msp {
namespace GL {

VulkanTexture2D::VulkanTexture2D():
	Texture(VK_IMAGE_VIEW_TYPE_2D)
{ }

void VulkanTexture2D::fill_image_info(void *ii) const
{
	const Texture2D &self = *static_cast<const Texture2D *>(this);

	VkImageCreateInfo *image_info = static_cast<VkImageCreateInfo *>(ii);
	image_info->imageType = VK_IMAGE_TYPE_2D;
	image_info->extent.width = self.width;
	image_info->extent.height = self.height;
	image_info->mipLevels = self.levels;
}

void VulkanTexture2D::sub_image(unsigned level, int x, int y, unsigned wd, unsigned ht, const void *data)
{
	const Texture2D &self = *static_cast<const Texture2D *>(this);

	auto level_size = self.get_level_size(level);
	synchronize(-1, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, (x==0 && y==0 && wd==level_size.x && ht==level_size.y));

	size_t data_size = wd*ht*get_pixel_size(storage_fmt);
	void *staging = device.get_transfer_queue().prepare_transfer(data_size,
		[this, level, x, y, wd, ht](VkCommandBuffer cmd_buf, VkBuffer staging_buf, size_t src_off){
			const VulkanFunctions &vk = device.get_functions();

			VkBufferImageCopy region = { };
			region.bufferOffset = src_off;
			region.imageSubresource.aspectMask = VK_IMAGE_ASPECT_COLOR_BIT;
			region.imageSubresource.mipLevel = level;
			region.imageSubresource.baseArrayLayer = 0;
			region.imageSubresource.layerCount = 1;
			region.imageOffset = { x, y, 0 };
			region.imageExtent = { wd, ht, 1 };
			vk.CmdCopyBufferToImage(cmd_buf, staging_buf, handle, VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 1, &region);
		});

	const char *src = static_cast<const char *>(data);
	copy(src, src+data_size, static_cast<char *>(staging));
}

Resource::AsyncLoader *VulkanTexture2D::load(IO::Seekable &, const Resources *)
{
	throw logic_error("Texture2D::load is unimplemented");
}

uint64_t VulkanTexture2D::get_data_size() const
{
	return 0;
}

void VulkanTexture2D::unload()
{
}

} // namespace GL
} // namespace Msp
