#ifndef MSP_GL_TEXTURE_BACKEND_H_
#define MSP_GL_TEXTURE_BACKEND_H_

#include <msp/core/noncopyable.h>
#include "handles.h"

namespace Msp {
namespace GL {

class Device;

class VulkanTexture: public NonCopyable
{
	friend class VulkanFramebuffer;
	friend class VulkanPipelineState;

protected:
	Device &device;
	VkImage handle = 0;
	VkImageView view_handle = 0;
	unsigned memory_id = 0;
	unsigned view_type;
	std::string debug_name;

	VulkanTexture(unsigned);
	VulkanTexture(VulkanTexture &&);
	~VulkanTexture();

	void allocate();
	virtual void fill_image_info(void *) const = 0;
	void require_swizzle() { }

	void generate_mipmap();

	void synchronize(int, unsigned, bool = false) const;

	void set_debug_name(const std::string &);
	void set_vulkan_object_names() const;
};

using TextureBackend = VulkanTexture;

} // namespace GL
} // namespace Msp

#endif
